

#include <Wire.h>                           //system library for I2CIO.cpp
#include <LiquidCrystal_I2C.h>              //library LiquidCrystal_I2C
LiquidCrystal_I2C lcd (0x27, 16, 2);        //to define object lcd 16x2 dimension                         

#define steptime 15

#include <Servo.h>
Servo servo_in;                                   // servo objects
Servo servo_out;

//entrance pin sensor
int trig_in=5;                                        
int echo_in=4;

//out pin sensor
int trig_out = 12;                                   
int echo_out = 11; 

//pin led
int green=6;                                            
int red =7;

//other variables
unsigned long duration_in = 0;           
unsigned long duration_out=0;
int distance_in = 0; 
int distance_out=0;
int distance_wall_in, distance_wall_out;
int entrance;
int out;
int free_spaces=8;
int angle;
boolean lowered_bar=1; //at the start, the bars are down

void setup ( )    {

    //display initialization
    lcd.init( );                                
    lcd.backlight( );
    lcd.setCursor(0,0);

    //declaration input or output pin type
    pinMode(echo_in, INPUT);
    pinMode(trig_in, OUTPUT); 
    pinMode(echo_out, INPUT);
    pinMode(trig_out, OUTPUT); 
    pinMode(red,OUTPUT);
    pinMode(green,OUTPUT);

    //pin and initial angular position of servos
    servo_in.attach(3);                       
    servo_out.attach(13);
    servo_in.write(90);
    servo_out.write(90);
 
    delay(100);
    
    servo_in.detach( );
    servo_out.detach( );

    //initial colors of traffic light
    digitalWrite(green, HIGH);
    digitalWrite(red,LOW); 
    
    
    
  
    //----distance between sensor_in and wall in front of it
    digitalWrite(trig_in, HIGH);
    delayMicroseconds(10); 
    digitalWrite(trig_in, LOW); 
    duration_in = pulseIn(echo_in, HIGH); 
    distance_wall_in = duration_in / 58; 
    
  //----distance between sensor_out and wall in front of it
    digitalWrite(trig_out, HIGH);
    delayMicroseconds(10); 
    digitalWrite(trig_out, LOW); 
    duration_out = pulseIn(echo_out, HIGH); 
    distance_wall_out = duration_out / 58;
    
    
    //----initial displayed message (in italian language)
    lcd.init( );
    lcd.print("   ATTENDERE  ");
    lcd.setCursor(0,1);
    lcd.print(" CONFIGURAZIONE ");
    delay(7000);
    _display( );
}

void loop ( )    {

    
    //--------------entrance managment---------------------------------
    digitalWrite(trig_in, HIGH);
    delayMicroseconds(10); 
    digitalWrite(trig_in, LOW); 
    duration_in = pulseIn(echo_in, HIGH); 
    distance_in = duration_in / 58; 
 
    delay(500);
    if(free_spaces>0)    {
        while(distance_wall_in - distance_in >=2 && distance_in>0)    {
           
            //bar up
            if(lowered_bar==1)   {
                servo_in.attach(3);
                for(angle = 90; angle > 1; angle--)    {
                    servo_in.write(angle);
                    delay(steptime);
                }
                servo_in.detach( );
                lowered_bar=0;   //the bar is not lowered anymore
            }

            entrance=1;
          
          
            digitalWrite(trig_in, HIGH);
            delayMicroseconds(10); 
            digitalWrite(trig_in, LOW); 
 
            duration_in = pulseIn(echo_in, HIGH); 
            distance_in = duration_in / 58; 
            
            delay(500);
       }
       delay(500);                                     
   
       //bar down
       if(lowered_bar==0)   {
           delay(1000);
           servo_in.attach(3);
           for(angle = 1; angle <= 90; angle++)    {
              servo_in.write(angle);
              delay(steptime);
           }
           servo_in.detach( );
           lowered_bar=1;  //entrance completed
       }
    }
   
    //---------counter managment---------------
    if (entrance)    {
        free_spaces--;
        entrance=0;
        _display( );
    }
    
    //-------------red led managment--------------
    if(free_spaces==0)    {
         digitalWrite(red,HIGH);
         digitalWrite(green,LOW);
    }
    
    //--------------------------exit managment-----------------------------
    digitalWrite(trig_out, HIGH);
    delayMicroseconds(10); 
    digitalWrite(trig_out, LOW); 
    duration_out = pulseIn(echo_out, HIGH); 
    distance_out = duration_out / 58; 
    
    delay(500);

    //bar up
    if(free_spaces<8)    {
        while(distance_wall_out - distance_out >=2 && distance_out >0)    {
           
           if(lowered_bar==1)  {
               servo_out.attach(13);
               for(angle = 90; angle >1; angle--)    {
                   servo_out.write(angle);
                   delay(steptime);
                }
                servo_out.detach( );
                lowered_bar=0;   //the bar is not lowered anymore
           }
           out=1;

           digitalWrite(trig_out, HIGH);
           delayMicroseconds(10); 
           digitalWrite(trig_out, LOW); 
           duration_out = pulseIn(echo_out, HIGH); 
           distance_out = duration_out / 58;
          
           delay(500);
       }
       delay(500);                                     

        //bar down
        if(lowered_bar==0)   {
            delay(1000);
            servo_out.attach(13);
            for(angle = 1; angle <=90; angle++)    {
                servo_out.write(angle);
                delay(steptime);
            }
            servo_out.detach( );
            lowered_bar=1;  //exit completed
        }
    }
  
    //-----counter managment--------------------
    if (out)    {
       free_spaces++;
       out=0;
       _display( );
    }
    
    //-----------------green led managment-------
     if(free_spaces>0)    {
         digitalWrite(green,HIGH);
         digitalWrite(red,LOW);
      }
}

//-----------------display managment (message displayed in italian language)------------
 void _display ( )    {
       lcd.init( );
       switch (free_spaces)    {    

         case 0:
          lcd.print(" POSTI ESAURITI ");
          lcd.setCursor(0,1);
          lcd.print("ACCESSO VIETATO ");
          break;

          case 1:                                                                     
          lcd.print(" 1 POSTO DISPON. ");
          lcd.setCursor(0,1);
          lcd.print(" ACCESSO LIBERO ");
          break;

           case 2:                                                                      
           lcd.print(" 2 POSTI DISPON. ");
           lcd.setCursor(0,1);
           lcd.print(" ACCESSO LIBERO ");
           break;

           case 3:                                                                     
           lcd.print(" 3 POSTI DISPON. ");
           lcd.setCursor(0,1);
           lcd.print(" ACCESSO LIBERO ");
           break;

          case 4:                                                                   
          lcd.print(" 4 POSTI DISPON. ");
          lcd.setCursor(0,1);
          lcd.print(" ACCESSO LIBERO ");
          break;

          case 5:                                                                   
          lcd.print(" 5 POSTI DISPON. ");
          lcd.setCursor(0,1);
          lcd.print(" ACCESSO LIBERO ");
          break;

          case 6:                                                                    
          lcd.print(" 6 POSTI DISPON. ");
          lcd.setCursor(0,1);
          lcd.print(" ACCESSO LIBERO ");
          break;

          case 7:                                                                    
          lcd.print(" 7 POSTI DISPON. ");
          lcd.setCursor(0,1);
          lcd.print(" ACCESSO LIBERO ");
          break;

          case 8:                                                                  
          lcd.print(" 8 POSTI DISPON. ");
          lcd.setCursor(0,1);
          lcd.print(" ACCESSO LIBERO ");
          break;
        }//end switch
}//end _display
